// import path from 'path'
import routes from "./routes"
// import log4js from 'log4js';
import baseconfig from "./config/base"
import config from "./config/index"

const Koa = require("koa")
const Router = require("koa-router")
const pino = require("pino")
const onerror = require("koa-onerror")
const logging = require("@kasa/koa-logging")
const requestId = require("@kasa/koa-request-id")
const cors = require("@koa/cors")
// const LOG = log4js.getLogger('file')

const app = new Koa()

app.use(cors())
baseconfig(app)
onerror(app)

// 日志格式化服务 序列化操作
const serializers = require("@kasa/koa-logging/lib/serializers")

type ResBodyInfo = {
    status: any
    code: any
    message: any
}
function resBodySerializer(param: ResBodyInfo) {
    const { status, code, message } = param
    const body = {
        status,
        message,
        code,
    }
    if (!code) {
        delete body.code
    }
    return body
}

function resSerializer(ctx: any) {
    return {
        statusCode: ctx.status,
        duration: ctx.duration,
        type: ctx.type,
        headers: (ctx.response || {}).headers,
        body: resBodySerializer(ctx.body || {}),
    }
}

// Logging to STDOUT/STDERR stream using Pino
// 记录日志等级 process.env.LOG_LEVEL
app.use(requestId())
app.use(
    logging({
        logger: pino({
            enabled: true,
            level: "warn", // 'info'
            serializers: {
                ...serializers,
                res: resSerializer,
            },
        }),
    })
)

app.use(async (ctx: any, next: any) => {
    const start = Date.now()
    await next()
    const ms = Date.now() - start
    console.log(`${ctx.method} ${ctx.url} - ${ms}ms`)
})

function handleError(err: any, ctx: any) {
    if (err) {
        console.log("应用错误：", err)
    }
    if (ctx) {
        console.log("错误信息：", JSON.stringify(ctx.onerror))
    }
}
app.on("error", handleError)

const router = new Router()
app.use(routes())
app.use(router.allowedMethods())
app.use(router.routes())

app.listen(config.app.port, () => {
    console.info(
        { event: "execute" },
        // `API server listening on ${config.app.host}:${config.app.port}, in ${config.app.env}`
        `API server listening on port:${config.app.port}, in ${config.app.env}`
    )
})

export default app
