import config from "./index"

const convert = require("koa-convert")
const Cors = require("@koa/cors")
const bodyParser = require("koa-bodyparser")
const Helmet = require("koa-helmet")
const respond = require("koa-respond")

const serve = require("koa-static")
const logger = require("koa-logger")

const views = require("koa-views")

const log4js = require("log4js")

export default function middleware(app: any) {
    // eslint-disable-next-line no-param-reassign
    app.proxy = true

    log4js.configure({
        // appenders: [
        //     { type: 'console' },
        //     { type: 'dateFile', filename: __dirname + '/../tmp/boilerplate.log' , "pattern":"-yyyy-MM-dd-hh.log","alwaysIncludePattern":false, category: 'file' }
        // ],
        appenders: { cheese: { type: "file", filename: "cheese.log" } },
        categories: { default: { appenders: ["cheese"], level: "error" } },
        replaceConsole: true,
    })
    app.use(convert(logger()))

    // 安全内容控制，添加常见的安全防护
    app.use(Helmet())
    app.use(Cors())
    app.use(
        bodyParser({
            enableTypes: ["json"],
            jsonLimit: "10mb",
            strict: true,
            onerror(err: any, ctx: any) {
                if (err) {
                    console.log(err)
                }
                ctx.throw("body parse error", 422)
            },
        })
    )

    app.use(respond())
    app.use(serve(`${__dirname}/../../public`))
    app.use(serve(`${__dirname}/../../dist`))
    app.use(views(`${__dirname}/../views`, { extension: "ejs" }))
    // eslint-disable-next-line no-param-reassign
    app.keys = ["superalsrk-session-key"]
    // eslint-disable-next-line no-param-reassign
    app.context.state = {}
    // eslint-disable-next-line no-param-reassign
    app.context.state = Object.assign(app.context.state, { config })
}
