import axios from "axios"
import FormData from "form-data"
import ResCtl from "../temporary_res/resinfo"
import { CommonHttpResult } from "../../utils/http"
import localLog, { localLogErr } from "../../utils/logUtil"
import { movieImageComposition, MovieImageInfo } from "./imgComb/movie"
import { tvSeriesComposition, TvSeriesImageInfo } from "./imgComb/tvseries"
import { youtubeImageComposition, YoutubeImageInfo } from "./imgComb/youtube"
import postUtil from "../../utils/post"
import "../../../types/global"

const cfgFileName = process.env.NODE_ENV_CFG || "development"

// eslint-disable-next-line import/no-dynamic-require
const dataJSON = require(`../../config/${cfgFileName}.json`)
localLog(
    "[server]-[config]-[db]-p2 env ",
    String(process.env.NODE_ENV),
    ", cfgFileName ",
    cfgFileName
)

const CONSTS_SHARE_TYPES = {
    NORMAL: "3",
    YOUITUBE: "4",
    YOUTUBE_CLIP: "5",
    TVS: "6",
    MOVIE: "11",
    SELF_CLIP: "16",
}

type TvSeriesInfo = {
    episodeCount: number
    isFinished: boolean
}
function buildTVSeries(tvSeriesInfo: TvSeriesInfo) {
    return `isFinished:${tvSeriesInfo.isFinished}, episodeCount:${tvSeriesInfo.episodeCount}`
}

type CombineImageInfo = {
    type: string
    id: string
    title: string
    description: string
    iconUrl: string
    lang: string
}
/**
 * 内部调用方法方法 (合成图片/保存图片/保存数据)
 * */
export const combineImageAndSaveInfoP2 = async (
    param: CombineImageInfo,
    ctx: any
): Promise<CommonHttpResult<any> | undefined> => {
    const { type, id, title, description, iconUrl, lang } = param
    localLog(
        "/res/shImg => combineImageAndSaveInfoP2 => ",
        JSON.stringify({
            type,
            id,
            title,
            description,
            iconUrl,
            lang,
        })
    )
    localLog(
        "[server]-[config]-[db]-p2 env ",
        String(process.env.NODE_ENV),
        ", cfgFileName ",
        cfgFileName
    )
    // 1. 如果已存在数据，则不用重复生成图片/保存图片/保存数据
    const resId = `tp${type}_${id}`
    console.log("电视剧11111111")
    const resInfo2 = await ResCtl.getResInfo(resId)
    if (resInfo2) {
        console.log("电视剧222222222")
        localLog("/res/shImg => db-item existed...")
        // 在后续进行判断
        if (type !== CONSTS_SHARE_TYPES.TVS) {
            return {
                Result: 1,
                success: true,
                code: 200,
                data: true,
            } as CommonHttpResult<boolean>
        }

        // 电视剧需要更新图片上的信息，ajax 获取，对比 description，相等 则不用再合成
        const resXX = await axios.get(
            `${dataJSON.metaImgActionUrl}/feed/sharePage/seasonDetail?seasonId=${id}&lang=id&countryCode=ID&region=ID`
        )
        const rltXXX = resXX.data
        const tvSeriesInfo = {
            episodeCount: rltXXX.data.episodeCount,
            isFinished: rltXXX.data.isFinished === 1,
        }
        const newDesc = buildTVSeries(tvSeriesInfo)
        console.log("7777777777777777777777777", resId, resInfo2.description2, newDesc)
        if (resInfo2.description2 === newDesc) {
            // 已经合成过了
            localLog("/res/shImg => tvseries db-item existed...")
            return {
                Result: 1,
                success: true,
                code: 1,
                data: true,
            } as CommonHttpResult<boolean>
        }

        localLog(
            "/res/shImg => combineImageAndSaveInfoP2 => CONSTS_SHARE_TYPES.TVS, need new combine."
        )
    } else {
        console.log("电视剧333333")
        localLog("/res/shImg => combineImageAndSaveInfoP2 => not existed.")
    }

    // 2. 图片合成
    let combInfo: MovieImageInfo | TvSeriesImageInfo | YoutubeImageInfo | undefined
    let newIconUrl = ""
    let fileName = ""
    let newTitle = ""
    let newDescription = ""
    let newDescription2 = ""
    if (id.indexOf("mo_") > -1 || type === CONSTS_SHARE_TYPES.NORMAL) {
        // 电影
        localLog("/res/shImg => movie...........................")
        const mvCombInfo = await movieImageComposition(id, lang)
        if (!mvCombInfo) return undefined
        newTitle = mvCombInfo.name
        newDescription = mvCombInfo.description
        newIconUrl = mvCombInfo.iconUrl
        if (!mvCombInfo.imgBytes || mvCombInfo.imgBytes.length === 0) return undefined // 无数据的情况处理
        combInfo = mvCombInfo

        // 构造fileName
        const filePathPcs = newIconUrl.split(".")
        const fileEnd = filePathPcs[filePathPcs.length - 1] || "png"
        fileName = `share_${id}.${fileEnd}`
    } else if (id.indexOf("tv_") > -1 || type === CONSTS_SHARE_TYPES.TVS) {
        // 电视剧
        localLog("/res/shImg => tvSeries...........................")
        const tvCombInfo = await tvSeriesComposition(id, lang)
        localLog("/res/shImg => tvSeries...........................")
        if (!tvCombInfo) return undefined
        newTitle = tvCombInfo.title
        newDescription = tvCombInfo.introduction
        newDescription2 = `isFinished:${tvCombInfo.isFinished}, episodeCount:${tvCombInfo.episodeCount}` // combInfo.description
        newIconUrl = tvCombInfo.iconUrl
        if (!tvCombInfo.imgBytes || tvCombInfo.imgBytes.length === 0) return undefined // 无数据的情况处理
        combInfo = tvCombInfo

        // 构造fileName
        const filePathPcs = newIconUrl.split(".")
        const fileEnd = filePathPcs[filePathPcs.length - 1] || "png"
        fileName = `tp${type}_${id}.${fileEnd}`
    } else {
        // youtube
        localLog("/res/shImg => youtube..........................")
        newTitle = title
        newDescription = description

        let tmpImgPrefix = ""
        if (process.env.NODE_ENV === "production") {
            tmpImgPrefix = "http://0.0.0.0:3012"
        } else {
            tmpImgPrefix = "http://127.0.0.1:3012"
        }
        localLog(
            "/res/shImg => tmpImgPrefix=========================================",
            tmpImgPrefix
        )
        const iconUrlPcs = iconUrl.split("?")
        const [newIconUrl2] = iconUrlPcs
        newIconUrl = newIconUrl2
        const ytbCombInfo = await youtubeImageComposition(newIconUrl, tmpImgPrefix, id)
        if (!ytbCombInfo) return undefined
        fileName = `share_${ytbCombInfo.name}`
        combInfo = ytbCombInfo
    }

    // 3. 保存合成后的图片
    const formData = new FormData()
    formData.append("key", dataJSON.uploadUrlSavedPath)
    formData.append("appType", dataJSON.productInfo.appType)
    formData.append("file", combInfo.imgBytes, (type ? `tp${type}_` : "") + fileName)
    const fileUploadApi = `${dataJSON.uploadUrlPrefix}/res/upload`

    localLog("/res/shImg => upload start =>", `${fileUploadApi}/res/upload`)
    const res = await axios.post(fileUploadApi, formData, {
        headers: formData.getHeaders(),
    })
    localLog("/res/shImg => upload end. =>", JSON.stringify(res.data))
    const rlt = res.data
    if (rlt.Result === 1 && rlt.data) {
        newIconUrl = rlt.data
    } else {
        // 上传失败? 先不管
        newIconUrl = "/img/meta/vidnow_share_imgs/defaultMetaVdl-id.png"
    }

    // 4. 数据保存到数据库
    // localLog('-----------------------------No data. Add it start:', {
    //   id: resId, // 应用来源 type + id: 'cont_' + key
    //   title: title.length > 200 ?  title.substring(0, 200): title,
    //   description: description.length > 1000 ?  description.substring(0, 1000): description,
    //   iconUrl: newIconUrl.length > 200 ?  newIconUrl.substring(0, 200): newIconUrl
    // })

    const resInfo = await ResCtl.getResInfo(resId)
    if (resInfo) {
        localLog("/res/shImg => go to delete.")
        const dataStr = await postUtil.getPostData(ctx)

        const data = JSON.parse(dataStr)
        ctx.body = await ResCtl.removeResInfo(data)
    }

    // let saveRlt = {}
    localLog(
        "/res/shImg => go to add start ...",
        JSON.stringify({
            isExistNewTitle: !!newTitle,
            isExistNewDescription: !!newDescription,
            isExistNewIconUrl: !!newIconUrl,
        })
    )
    if (newIconUrl.indexOf("?") > -1) {
        newIconUrl += `&rnd=${Math.floor(Math.random() * 10000)}`
    } else {
        newIconUrl += `?rnd=${Math.floor(Math.random() * 10000)}`
    }
    const saveRlt = await ResCtl.saveResInfo({
        id: resId, // 应用来源 type + id: 'cont_' + key
        title: newTitle && newTitle.length > 200 ? newTitle.substring(0, 200) : newTitle,
        description:
            newDescription && newDescription.length > 1000
                ? newDescription.substring(0, 1000)
                : newDescription,
        iconUrl: newIconUrl && newIconUrl.length > 200 ? newIconUrl.substring(0, 200) : newIconUrl,
        description2: newDescription2 || "",
    })
    const returnRlt: CommonHttpResult<any> = {
        ...saveRlt,
    }
    localLog("/res/shImg => go to add end ...", JSON.stringify(saveRlt))
    return returnRlt
}

/**
 * 接口: http调用方法 (合成图片/保存图片/保存数据)
 */
export const combineImageAndSaveInfo = async (ctx: any): Promise<undefined> => {
    localLog(
        `/res/shImg => combineImageAndSaveInfo start => curCombImgConcurrentNum:${global.CONNECTION_CUR_COUNT}; combImgConcurrentNum:${global.CONNECTION_MAX_COUNT}`
    )

    if (global.CONNECTION_CUR_COUNT >= global.CONNECTION_MAX_COUNT) {
        ctx.body = {
            Result: 0,
            success: false,
            code: 0,
            message: `too much Concurrent Num: ${global.CONNECTION_CUR_COUNT}`,
        } as CommonHttpResult<any>
        return undefined
    }
    global.CONNECTION_CUR_COUNT += 1

    const { type, id, title, description, iconUrl, lang } = ctx.request.body

    localLog(
        "/res/shImg => combineImageAndSaveInfo => ",
        JSON.stringify({
            type,
            id,
            title,
            description,
            iconUrl,
            lang,
        })
    )

    let l = lang
    if (!l) {
        l = ctx.query.l
    }

    let res: CommonHttpResult<any> | undefined
    // setTimeout(async () => {
    try {
        res = await combineImageAndSaveInfoP2(
            {
                type,
                id,
                title,
                description,
                iconUrl,
                lang: l,
            },
            ctx
        )
    } catch (ex: any) {
        res = {
            Result: 0,
            success: false,
            code: 0,
            message: String(ex),
        } as CommonHttpResult<any>
        localLogErr(`/res/shImg => combineImageAndSaveInfo error=>${ex}`)
    }
    global.CONNECTION_CUR_COUNT -= 1
    // })
    ctx.body = res

    return undefined
}

// console.log('test1111111111=>',
//  getFileInfoOfFilePath("F:\\01_projects\\02_vdmaster-h5(in_testing_or_deploying)\\
//    vidnow_share_msa\\webp_combine_test\\src\\public\\img\\share\\test4.png"))
// console.log('test2222222222=>', getFileInfoOfFilePath("http://localhost:3012/img/test4.webp"))
// console.log('test3333333333=>', getFileInfoOfFilePath("https://androidmaster-dn1.s3-us-west-1.amazonaws.com/shareRes/composedImg/share_mo_20201102_1166.jpg?rnd=3453"))
