import GaussianImgUtil from "../../../utils/gaussianImgUtil"
import localLog from "../../../utils/logUtil"

const axios = require("axios")

const path = require("path")
// const fs = require("fs")

const cfgFileName = process.env.NODE_ENV_CFG || "development"

// eslint-disable-next-line import/no-dynamic-require
const dataJSON = require(`../../../config/${cfgFileName}.json`)
localLog(
    "[server]-[config]-[db]-p2 env ",
    String(process.env.NODE_ENV),
    ", cfgFileName ",
    cfgFileName
)
const { createCanvas, loadImage } = require("canvas")

export type TvSeriesImageInfo = {
    imgBytes?: Buffer
    actorList: string
    countryList: string
    director: string
    episodeCount: number
    genre: string
    horizontalIcon: string
    iconUrl: string
    introduction: string
    isFinished: number
    score: number
    seasonId: string
    title: string
}
/**
 * 电视剧图片合成
 * @param { 季Id } episodeId
 * @param { 语言 } lang
 * @returns
 */
export async function tvSeriesComposition(
    episodeId: string,
    lang: string
): Promise<TvSeriesImageInfo | undefined> {
    localLog("tvSeriesComposition start:")

    // 1. 获取源图片
    let srcImgUrl = null
    const res = await axios.get(
        `${dataJSON.metaImgActionUrl}/feed/sharePage/seasonDetail?seasonId=${episodeId}&lang=id&countryCode=ID&region=ID`
    )
    const rlt = res.data

    console.log(
        "rlt=>",
        `${dataJSON.metaImgActionUrl}/feed/sharePage/seasonDetail?seasonId=${episodeId}&lang=id&countryCode=ID&region=ID`,
        rlt
    )
    localLog(`tvSeriesComposition query info => rlt ${JSON.stringify(rlt)}`)
    const tvSeriesInfo: TvSeriesImageInfo = {
        actorList: "",
        countryList: "",
        director: "",
        episodeCount: 0,
        genre: "",
        horizontalIcon: "",
        iconUrl: "",
        introduction: "",
        isFinished: 1,
        score: 0,
        seasonId: "",
        title: "",
    }
    if (rlt.Result === 1 && rlt.data && rlt.data.iconUrl) {
        localLog(
            "correct mv info [",
            typeof rlt.data.actorList,
            ", ",
            rlt.data.actorList,
            "], [",
            typeof rlt.data.countryList,
            rlt.data.countryList
        )
        srcImgUrl = rlt.data.iconUrl
        tvSeriesInfo.actorList = rlt.data.actorList ? JSON.parse(rlt.data.actorList).join(",") : ""
        tvSeriesInfo.title = rlt.data.title
        if (rlt.data.countryList && rlt.data.countryList.indexOf("[") > -1) {
            tvSeriesInfo.countryList = JSON.parse(rlt.data.countryList).join(",")
        } else {
            tvSeriesInfo.countryList = rlt.data.countryList || ""
        }

        tvSeriesInfo.genre = rlt.data.genre ? JSON.parse(rlt.data.genre).join(",") : ""
        tvSeriesInfo.episodeCount = rlt.data.episodeCount
        tvSeriesInfo.isFinished = rlt.data.isFinished
        tvSeriesInfo.introduction = rlt.data.introduction
        localLog(`getFeedMovieDetail str.length =>${JSON.stringify(rlt.data).length}`)
    } else {
        localLog("wrong tv info")
        return Promise.resolve(undefined)
    }

    // 2. 初始化合成器
    const tarWidth = 806
    const tarHeight = 390

    const image = await loadImage(srcImgUrl)
    const { width, height } = image // 获取图片宽高
    const t_canvas = createCanvas(tarWidth, tarHeight) // 按照微信官方要求，长宽比5:4
    const context = t_canvas.getContext("2d")

    // 3. 使用源图片合成合成新图片
    // 3.1 绘制模糊图
    await GaussianImgUtil.setGaussianBlurCanvas3({
        cvsContext: context,
        tarWidth,
        tarHeight,
        image,
    })
    context.fillStyle = "rgba(0,0,0,.3)"
    context.fillRect(0, 0, tarWidth, tarHeight)
    // 3.2 绘制清晰图
    // context.fillStyle = "rgba(0,0,0,255)"context.globalAlpha = 1 //调节透明度
    context.globalAlpha = 1 // 调节透明度
    if (width / height < 234 / 333) {
        // 纵图
        const dw_img2 = Math.floor((width * 333) / height)
        const dh_img2 = 333
        const dx_img2 = 50
        const dy_img2 = 31
        context.drawImage(image, dx_img2, dy_img2, dw_img2, dh_img2)
    } else {
        // 横图
        const rate = 333 / height
        const dw_img2 = Math.floor(234 / rate)
        const dh_img2 = Math.floor(333 / rate)
        const sx_img2 = -1 * Math.floor((234 - width * rate) / rate / 2)
        const sy_img2 = 0
        context.drawImage(image, sx_img2, sy_img2, dw_img2, dh_img2, 50, 31, 234, 333)
    }

    // 黑色透明图层
    // 3.3 打印更新的集数
    let episodeStr = ""
    if (tvSeriesInfo.isFinished) {
        episodeStr = "END"
    } else {
        episodeStr = `Ep. ${tvSeriesInfo.episodeCount}`
    }
    context.font = "22px bold" // 统一使用微软雅黑字体  "Microsoft YaHei"
    context.fillStyle = "#FFFFFF"
    const updateStrRect = context.measureText(episodeStr)
    const updateStrWidth = updateStrRect.width // title
    const offLR = 20
    const updateStartPos = 270 - updateStrWidth - offLR > 0 ? 270 - updateStrWidth - offLR : 0

    // 背景
    if (updateStrWidth > 80) {
        const scorebgImgPath = path.resolve(__dirname, "../../../../public/img/share/score_bg2.png")
        const scorebgImg = await loadImage(scorebgImgPath)
        context.drawImage(
            scorebgImg,
            updateStartPos,
            315,
            updateStrWidth + offLR,
            Math.floor(scorebgImg.height * 1.5)
        )
    } else {
        const scorebgImgPath = path.resolve(__dirname, "../../../../public/img/share/score_bg3.png")
        const scorebgImg = await loadImage(scorebgImgPath)
        context.drawImage(
            scorebgImg,
            updateStartPos,
            315,
            updateStrWidth + offLR,
            Math.floor(scorebgImg.height * 1.5)
        )
    }
    // 集数
    context.fillText(episodeStr, updateStartPos + Math.floor(offLR / 2), 341)

    // 3.4 绘制logo
    const logoImgPath = path.resolve(__dirname, "../../../../public/img/share/logo.png")
    const logoImg = await loadImage(logoImgPath)
    context.drawImage(logoImg, 309, 46, logoImg.width, logoImg.height)

    // 3.5 打印title
    context.font = "30px bold Arial" // 统一使用微软雅黑字体  "Microsoft YaHei"
    context.fillStyle = "white"
    const nameTxtOffX = 309
    const nameTxtWidth = context.measureText(tvSeriesInfo.title).width // title
    const nameTxtSubLen = Math.ceil(
        ((tarWidth - nameTxtOffX - 48) / nameTxtWidth) * tvSeriesInfo.title.length
    )
    if (nameTxtSubLen < tvSeriesInfo.title.length) {
        context.fillText(`${tvSeriesInfo.title.substr(0, nameTxtSubLen)}...`, nameTxtOffX, 140)
    } else {
        context.fillText(tvSeriesInfo.title, nameTxtOffX, 140)
    }

    // 3.6 打印评分
    const starbgImgPath = path.resolve(__dirname, "../../../../public/img/share/star.png")
    const starbgImg = await loadImage(starbgImgPath)
    context.drawImage(
        starbgImg,
        309,
        166,
        Math.floor(starbgImg.width * 0.9),
        Math.floor(starbgImg.height * 0.9)
    )

    context.font = "30px bold Arial" // 统一使用微软雅黑字体  "Microsoft YaHei"
    context.fillStyle = "#ffffffb2"
    context.fillText("7.8", 350, 195) // score

    // 3.7 打印类型
    // context.font = '24px "Microsoft YaHei"' // 统一使用微软雅黑字体
    const genreTxtOffX = 309
    context.font = "30px bold 'Microsoft YaHei'" // 统一使用微软雅黑字体  "Microsoft YaHei"
    tvSeriesInfo.genre = "drama America"
    const genreTxtWidth = context.measureText(tvSeriesInfo.genre).width
    const genreTxtSubLen = Math.ceil(
        ((tarWidth - genreTxtOffX - 48) / genreTxtWidth) * tvSeriesInfo.genre.length
    )
    if (genreTxtSubLen < tvSeriesInfo.genre.length) {
        context.fillText(`${tvSeriesInfo.genre.substr(0, genreTxtSubLen)}...`, genreTxtOffX, 235)
    } else {
        context.fillText(tvSeriesInfo.genre, genreTxtOffX, 235)
    }

    // 3.8 打印演员: 42 chars limit, 两行，第二行省略号
    // context.font = '24px "Microsoft YaHei"' // 统一使用微软雅黑字体
    const actor = `${
        lang === "id" ? "Aktor: " : "Actor: "
    }Naseeruddin Shah Vasundhara Das Parvin Dabas`
    let actorPcs = actor.split(" ")
    for (let i = 0; i < actorPcs.length; i += 1) {
        const pcs2 = actorPcs[i].split(",")
        actorPcs[i] = pcs2.join(", ")
    }
    actorPcs = actorPcs.join(" ").split(" ") // 分词换行处理
    // 第一行
    let actorTxtOffXP1 = 309
    let idx = 0 // 最多1000个词
    while (actorPcs.length > 0 && idx < 1000) {
        const word = `${actorPcs.shift()} `
        const wordW = context.measureText(word).width
        if (actorTxtOffXP1 + wordW > tarWidth - 48) {
            break
        } else {
            context.fillText(word, actorTxtOffXP1, 285)
        }

        actorTxtOffXP1 += wordW
        idx += 1
    }
    // 第二行
    let actorTxtOffXP2 = 309
    idx = 0 // 最多1000个词
    while (actorPcs.length > 0 && idx < 1000) {
        const word = `${actorPcs.shift()} `
        const wordW = context.measureText(word).width
        if (actorTxtOffXP2 + wordW > tarWidth - 48) {
            context.fillText("...", actorTxtOffXP2, 325)
            break
        } else {
            context.fillText(word, actorTxtOffXP2, 325)
        }

        actorTxtOffXP2 += wordW
        idx += 1
    }

    // 4. 生成文件字节流
    const file = t_canvas.toDataURL("image/png")
    const data2 = file.replace(/^data:image\/\w+;base64,/, "")
    const imgBytes = Buffer.from(data2, "base64")

    // // //本地保存，测试代码
    // const dataBuffer = new Buffer(data2, 'base64');
    // fs.writeFile(path.resolve(__dirname, '../../../../public/img/share/out.png'), dataBuffer, function(err){
    //   if (err) {
    //     console.log('保存失败！', err)
    //   } else {
    //     console.log("保存成功！");
    //   }
    // });

    localLog(`tvSeriesComposition end.${imgBytes.length}`)
    return {
        imgBytes,
        ...tvSeriesInfo,
    }
}
