import path from "path"
import fs from "fs"
import { createCanvas, loadImage } from "canvas"
import localLog, { localLogErr } from "../../../utils/logUtil"

// const axios = require("axios-https-proxy-fix")

const webp = require("webp-converter")
const request = require("request")
// const pixelUtil = require("pixel-util")

console.log("NODE_ENV______________________", process.env.NODE_ENV, process.env.NODE_ENV_CFG)

/**
 * 从path中解析文件信息
 */
function getFileInfoOfFilePath(filePath: string) {
    let pathPcs = filePath.split("\\")
    if (pathPcs.length < 2) {
        pathPcs = filePath.split("/")
    }
    const fileNameEnd = pathPcs[pathPcs.length - 1].split("?")[0]

    const fileNameEndPcs = fileNameEnd.split(".")
    const end = fileNameEndPcs[fileNameEndPcs.length - 1]
    const fileName = fileNameEndPcs.splice(0, fileNameEndPcs.length - 1).join(".")

    return {
        fileNameEnd,
        fileName,
        end,
    }
}

/**
 * 将webp下载 并转化为 png/jpg格式的本地图片
 * @param { 源url/源path } srcPath
 * @param { 临时文件url的前缀 } tmpImgPrefix
 * @returns 临时文件的url: tmpImgUrl; 临时文件的local_path: tmpImgLocPath;
 */
async function transfer_webp_tojpg(
    srcPath: string,
    tmpImgPrefix: string
): Promise<{
    tmpImgUrl: string
    tmpImgLocPath: string
}> {
    const tmpSubPath = "/img/share/"
    const newType = "png"

    const srcFileName = getFileInfoOfFilePath(srcPath).fileNameEnd
    const tmpFileName = srcFileName.replace(/.webp$/, `.${newType}`)

    // tmp local path
    const tmpImgLocPath = path.resolve(__dirname, `../../../../public${tmpSubPath}${tmpFileName}`)
    // tmp url
    const tmpImgUrl = tmpImgPrefix + tmpSubPath + tmpFileName
    // localLog('transfer_webp_tojpg=>', {
    //   srcPath,
    //   tmpImgLocPath,
    //   tmpImgUrl
    // })
    let srcLocPath = ""
    // 本地保存source文件
    if (srcPath.indexOf("http") > -1) {
        srcLocPath = path.resolve(__dirname, `../../../../public${tmpSubPath}${srcFileName}`)
        // srcLocPath = path.resolve(__dirname, `../../../../public${tmpSubPath}mqdefault2.webp`)
        // http远程文件
        const stream = fs.createWriteStream(srcLocPath)
        return new Promise((resolve) => {
            request(srcPath)
                .pipe(stream)
                .on("close", async () => {
                    localLog(`文件[${srcPath}]下载完毕`)
                    const returnData = await transfer_webp_tojpg_p2(
                        srcPath,
                        srcLocPath,
                        tmpImgUrl,
                        tmpImgLocPath
                    )
                    localLog("i have saved remoted img.")
                    resolve(returnData)
                })
        })

        // console.log("wwwwwwwwwwwwwwwww", srcLocPath)
        // const returnData = await transfer_webp_tojpg_p2(
        //     srcPath,
        //     srcLocPath,
        //     tmpImgUrl,
        //     tmpImgLocPath
        // )
        // localLog("i have saved remoted img.")
        // return Promise.resolve(returnData)
    }

    srcLocPath = srcPath

    const returnData = await transfer_webp_tojpg_p2(srcPath, srcLocPath, tmpImgUrl, tmpImgLocPath)

    // console.log("transfer_webp_tojpg returnData =>", returnData)
    return Promise.resolve(returnData)
}

async function transfer_webp_tojpg_p2(
    srcPath: string,
    srcLocPath: string,
    tmpImgUrl: string,
    tmpImgLocPath: string
): Promise<{ tmpImgUrl: string; tmpImgLocPath: string }> {
    // 本地转换生成tmp文件
    localLog(`try do next.${srcLocPath}`)
    // console.log("rrrrrrrrrrrrrrrrrrrr", srcLocPath)
    // await Promise.resolve((resolve) => {
    //     fs.open(srcLocPath, "r", (err, data) => {
    //         console.log("eeeeeeeeeee", err, data)
    //         resolve(data)
    //     })
    // })
    // console.log("==============================", resxxx)
    // fs.readFileSync(srcLocPath, "binary")
    // let testFile = await Promise.resolve((resolve) => {
    //     const arr: Buffer[] = []
    //     const rs = fs.createReadStream(srcLocPath)
    //     rs.on("data", (chunk: Buffer) => {
    //         arr.push(chunk)
    //     })
    //     rs.on("end", (chunk: Buffer) => {
    //         arr.push(chunk)
    //         console.log("7777777777777", JSON.stringify(arr))
    //     })
    //     rs.on("error", (err: any) => {
    //         console.log(err)
    //     })
    // })

    // const error = await fs.readFileSync(srcLocPath, "binary")
    // if (error) {
    //     // console.log(`ERROR:${error}`)
    //     console.log("sssssssssssssssssss")

    //     return {
    //         tmpImgUrl: "",
    //         tmpImgLocPath: "",
    //     }
    // }
    // console.log("ttttttttttttttttttt")

    const transferRlt = await webp.dwebp(srcLocPath, tmpImgLocPath, "-o", "-v")
    localLog(`i have finished dwebp action.${transferRlt}`)
    if (srcPath.indexOf("http") > -1) {
        // 删除http远程文件下载的source文件
        await fs.unlinkSync(srcLocPath)
    }
    localLog("==============================")

    return {
        // buffer: newFileBuffer,
        /** 临时文件的url */
        tmpImgUrl,
        /** 临时文件的local path */
        tmpImgLocPath,
    }
}

// /**
//  * Remote文件保存为本地文件，然后生成对应的本地url
//  * @param { Remote文件Url } srcPath
//  * @param { 本地url前缀 } tmpImgPrefix
//  */
// export function saveRemoteImg(srcPath, tmpImgPrefix) {
//   let tmpSubPath = '/img/share/'
//   let srcFileName = getFileInfoOfFilePath(srcPath).fileNameEnd
//   let srcLocPath = path.resolve(__dirname, '../../../../public' + tmpSubPath + srcFileName)
//   let srcLocUrl = tmpImgPrefix + tmpSubPath + srcFileName
//   let stream = fs.createWriteStream(srcLocPath);
//   return new Promise((resolve, reject) => {
//     request(srcPath).pipe(stream).on("close", async function (err) {
//       console.log("文件[" + srcPath + "]下载完毕");
//       fs.readFile(srcLocPath, async function (error, data) {
//         if(error) {
//           reject({
//             message: String(error)
//           })
//         }
//         else {
//           resolve({
//             srcLocUrl,
//             buffer: data,
//             srcLocPath
//           })
//         }
//       })
//     })
//   })
// }

export type YoutubeImageInfo = {
    imgBytes?: Buffer
    name?: string
    tmpImgUrl: string
    tmpImgLocPath: string
}

/**
 * youtube图片合成
 * @param { 源文件url } srcImgUrl
 * @param { 源文件url前缀 } tmpImgPrefix
 * @param { 文件Id } fileId
 * @returns imgBytes: buffer数组; name: 新文件的文件名
 */
export async function youtubeImageComposition(
    srcImgUrl: string,
    tmpImgPrefix: string,
    fileId: string
): Promise<YoutubeImageInfo | undefined> {
    localLog("youtubeImageComposition start:")

    // 1. 获取源图片
    localLog("youtubeImageComposition => 1. load source image.")
    let image = null
    let ytbRlt: YoutubeImageInfo = {
        tmpImgUrl: "",
        tmpImgLocPath: "",
    }
    const newPrefix =
        process.env.NODE_ENV === "development"
            ? "http://139.59.194.229:10926/"
            : "http://127.0.0.1:10926/"
    const oldSrcImgUrl = srcImgUrl.replace("https://i.ytimg.com/", newPrefix)
    if (getFileInfoOfFilePath(oldSrcImgUrl).end === "webp") {
        // 图片格式转化
        ytbRlt = await transfer_webp_tojpg(oldSrcImgUrl, tmpImgPrefix)
        localLog(
            "srcImgUrl is end with webp. created from ",
            oldSrcImgUrl,
            " to ",
            ytbRlt.tmpImgUrl
        )
        const newSrcImgUrl = ytbRlt.tmpImgUrl
        console.log("MMMMMMMMMMMMMMMMMMMMMMMMM newSrcImgUrl ", ytbRlt)
        image = await loadImage(newSrcImgUrl)
        // await fs.unlinkSync(ytbRlt.tmpImgLocPath); //临时图片，加载完成后删除
        localLog("youtubeImageComposition webp loaded:", newSrcImgUrl, JSON.stringify(ytbRlt))
    } else {
        localLog("srcImgUrl is not end with webp.", oldSrcImgUrl)
        // const {
        //   srcLocUrl,
        //   srcLocPath
        // } = await saveRemoteImg(srcImgUrl, tmpImgPrefix) //需要保存到本地再加载
        // console.log('srcImgUrl is saved in loc.')
        // image = await loadImage(srcLocUrl)
        try {
            // 知识从 https://www.modb.pro/db/323187 中获取
            image = await loadImage(oldSrcImgUrl, {
                timeout: 5000,
                Headers: {
                    userAgent:
                        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.4951.64 Safari/537.36 Edg/101.0.1210.53",
                    accept: "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
                    acceptEncoding: "gzip, deflate, br",
                    acceptLanguage: "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
                    // sec-ch-ua: " Not A;Brand";v="99", "Chromium";v="101", "Microsoft Edge";v="101"
                    // sec-ch-ua-mobile: ?0
                    // sec-ch-ua-platform: "Windows"
                    // sec-fetch-dest: document
                    // sec-fetch-mode: navigate
                    // sec-fetch-site: none
                    // sec-fetch-user: ?1
                    // upgrade-insecure-requests: 1
                    // user-agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.4951.64 Safari/537.36 Edg/101.0.1210.53
                },
            })
            // fs.unlinkSync(srcLocPath)
            localLog("youtubeImageComposition normal loaded:", srcImgUrl)
        } catch (ex) {
            localLogErr("youtubeImageComposition normal loaded:", JSON.stringify(ex))
            return undefined
        }
    }

    // 2. 初始化合成器
    localLog("youtubeImageComposition => 2. canvas inited")
    const { width, height } = image // 获取图片宽高
    const tarWidth = 806
    const tarHeight = 390
    const t_canvas = createCanvas(tarWidth, tarHeight) // 按照微信官方要求，长宽比5:4
    const context = t_canvas.getContext("2d")

    // 3. 使用源图片截取生成新图片
    localLog("youtubeImageComposition => 3. create new image by source image")
    let sw = width
    let sh = height
    if (width / height > tarWidth / tarHeight) {
        // 宽度过大
        localLog("t0")
        sw = Math.floor((width * tarHeight) / height)
        sh = height
    } else {
        // 高度过大
        localLog("t1")
        sw = tarWidth
        sh = Math.floor((height * tarWidth) / width)
    }
    const sx = -1 * Math.floor((tarWidth - sw) / 2)
    const sy = -1 * Math.floor((tarHeight - sh) / 2)
    context.drawImage(image, sx, sy, width, height, 0, 0, sw, sh)
    // 4. 生成文件字节流
    localLog("youtubeImageComposition => 4. Buffer for formdata is created.")
    const file = t_canvas.toDataURL("image/png")
    const data2 = file.replace(/^data:image\/\w+;base64,/, "")
    // 5. 本地保存再压缩
    localLog("youtubeImageComposition => 5. local save and compress.")
    let imgBytes = Buffer.from(data2, "base64")
    if (getFileInfoOfFilePath(oldSrcImgUrl).end === "webp") {
        // web需要 进行再压缩
        // 本地保存 png/jpg
        await fs.writeFileSync(ytbRlt.tmpImgLocPath, imgBytes)
        // png转webp
        const t_newEnd = getFileInfoOfFilePath(ytbRlt.tmpImgLocPath).end
        const tmpFilePath_Webp = ytbRlt.tmpImgLocPath.replace(t_newEnd, "webp")
        await webp.cwebp(ytbRlt.tmpImgLocPath, tmpFilePath_Webp, "-q 80", "-v")
        // 删除不再使用的jpg/png文件
        await fs.unlinkSync(ytbRlt.tmpImgLocPath)
        // 读取新的webp文件
        imgBytes = fs.readFileSync(tmpFilePath_Webp)
        // 删除不再使用的webp文件
        await fs.unlinkSync(tmpFilePath_Webp)
    }
    // fs.writeFile(tmpFilePath_Webp, dataBuffer, function(err){
    //   if (err) {
    //     console.log('保存失败！', err)
    //   } else {
    //     console.log("保存成功！");
    //   }
    // });
    localLog("youtubeImageComposition end.")
    return {
        imgBytes,
        name: String(`${fileId}.${getFileInfoOfFilePath(oldSrcImgUrl).end}`),
        tmpImgUrl: ytbRlt.tmpImgUrl,
        tmpImgLocPath: ytbRlt.tmpImgLocPath,
    }
}
