// 获取分享链接
import H5Tracer from "../../utils/H5Tracer"
import { CONSTS_MIXED_WORDS } from "../../constant"
import { AddConfusion } from "../../utils/obfuscator"
import restUtil, { HttpHeadOption } from "../../utils/restUtil"
import mimeTransfer from "../../utils/mime"
import localLog, { localLogErr } from "../../utils/logUtil"
import { CampaignInfo, ShareInfo } from "./types"
import { CommonHttpResult } from "../../utils/http"

const cfgFileName = process.env.NODE_ENV_CFG || "development"
const axios = require("axios")
const cheerio = require("cheerio")

// eslint-disable-next-line import/no-dynamic-require
const dataJSON = require(`../../config/${cfgFileName}.json`)

localLog(
    "[server]-[config]-[db]-p2 env ",
    String(process.env.NODE_ENV),
    ", cfgFileName ",
    cfgFileName
)
/**
 * 获取分享链接(旧-已作废)
 */
export const getShareLinkByPidAndNumber = async (ctx: any) => {
    localLog(
        "==================== getShareLinkByPidAndNumber req.headers.referer =>",
        ctx.req.headers.referer
    )
    const { query } = ctx.request
    if (!query || !query.campaignPid) {
        ctx.send(200, {
            code: 0,
            message: "campaignPid must provide",
        })
        return
    }
    if (!query.shareNumber) {
        ctx.send(200, {
            code: 0,
            message: "shareNumber must provide",
        })
        return
    }

    let { campaignPid, shareNumber } = query
    if (process.env.NODE_ENV === "development") {
        const codeObj = transferDnOld2NewCode({
            campaignPid,
            shareNumber,
        })
        localLog(
            "getShareLinkByPidAndNumber dn => old code to newCode:",
            JSON.stringify({ campaignPid, shareNumber }),
            ",",
            JSON.stringify(codeObj)
        )
        campaignPid = codeObj.campaignPid
        shareNumber = codeObj.shareNumber
    }

    const { productInfo } = dataJSON
    const { campaignInfo, shareInfo } = getShareInfo(campaignPid, shareNumber)

    if (!campaignInfo) {
        ctx.send(500, {
            message: "campaignInfo is not exist.",
        })
        return
    }
    if (!shareInfo) {
        ctx.send(500, {
            message: "shareInfo is not exist.",
        })
        return
    }

    const {
        // _id,
        campaignCode,
    } = campaignInfo

    const { shareCode, shDomainSet } = shareInfo
    const { domainSet } = productInfo
    // 随机数 命中概率要重新计算一下
    let prefix = ""
    let domainName = ""
    if (shDomainSet && shDomainSet?.length > 0) {
        localLog(
            "getShareLinkByPidAndNumber environment in shDomainSet: ",
            String(process.env.NODE_ENV),
            " ",
            JSON.stringify(shDomainSet)
        )
        prefix = shDomainSet[Math.round(Math.random() * (shDomainSet.length - 1) * 0.99)]
        domainName = prefix.replace(/http(s):\/\//, "")
    } else if (domainSet.length > 0) {
        localLog(
            "getShareLinkByPidAndNumber environment in domainSet: ",
            String(process.env.NODE_ENV),
            " ",
            domainSet
        )
        prefix = domainSet[Math.round(Math.random() * (domainSet.length - 1) * 0.99)]
        domainName = prefix.replace(/http(s):\/\//, "")
    } else if (ctx.request.header.host) {
        const protocol =
            ctx.req.connection.encrypted || ctx.request.headers.protocol ? "https://" : "http://"
        prefix = protocol + ctx.request.header.host
        domainName = ctx.request.header.host
    } else {
        ctx.send(200, {
            code: 0,
            message: "DomainSet is not configured.",
        })
        return
    }

    // 混淆sh路径
    const words = CONSTS_MIXED_WORDS
    let sharePath = "s2h"
    for (let i = 0; i < 2; i += 1) {
        // 增加两个随机字符
        const rand = words[Math.floor(Math.random() * words.length)]
        sharePath += rand
    }

    // 混淆vid
    let videoCode = ""
    for (let i = 0; i < 2; i += 1) {
        // 增加两个随机字符
        const rand = words[Math.floor(Math.random() * words.length)]
        videoCode += rand
    }

    const campaignCodePath = AddConfusion(campaignCode)
    const shareCodePath = AddConfusion(shareCode)
    // console.log('sharePath---------', sharePath, campaignCode, campaignCodePath, shareCode, shareCodePath)

    ctx.body = {
        Result: 1,
        success: true,
        code: 1,
        data: {
            shareLink: `${prefix}/${sharePath}/${campaignCodePath}/${shareCodePath}/${videoCode}.html`,
            sharePathLink: `/${sharePath}/${campaignCodePath}/${shareCodePath}/${videoCode}.html`,
            domainName,
            campaignCode,
            shareCode,
            // campaignRecordId: _id,
            shareNumber,
        },
    } as CommonHttpResult<any>
}

/**
 * 接口: 获取分享链接(新)
 */
export const getShareLinkExtByPidAndNumber = async (ctx: any) => {
    localLog(
        "getShareLinkExtByPidAndNumber=>",
        JSON.stringify({
            query: ctx.query,
            body: ctx.request.body,
        })
    )
    const { shareNumber } = ctx.query
    const { campaignPid } = ctx.query
    const { shareInfo } = getShareInfo(campaignPid, shareNumber)
    localLog(
        "[controllers]-[campaign]-[index] getShareLinkExtByPidAndNumber => ",
        JSON.stringify(shareInfo)
    )
    const { type } = shareInfo as ShareInfo
    const { id, title, description, iconUrl, lang } = ctx.request.body

    // 图片合成与数据存储
    if (id) {
        // 同步执行 图片合成 与 额外信息存储
        try {
            const res = await axios.post(`${dataJSON.combineImageAndSaveInfoPrefix}/res/shImg`, {
                type,
                id,
                title,
                description,
                iconUrl,
                lang,
            })
            localLog("combine img result => ", JSON.stringify(res.data))
        } catch (ex) {
            localLogErr(JSON.stringify(ex))
        }
    }
    await getShareLinkByPidAndNumber(ctx)
}

/**
 * 接口: 获取ip与区域信息
 * */
export const getIpRegion = async (ctx: any) => {
    const { req } = ctx
    const region = {
        country: "",
        province: "",
        city: "",
        street: "",
        detailAddr: "",
    }

    let ip = ""
    const ipAddr =
        req.headers["x-forwarded-for"] || // 判断是否有反向代理 IP
        req.connection.remoteAddress || // 判断 connection 的远程 IP
        req.socket.remoteAddress || // 判断后端的 socket 的 IP
        req.connection.socket.remoteAddress
    const ipMatches = ipAddr.match(/[0-9]{1,}.[0-9]{1,}.[0-9]{1,}.[0-9]{1,}/)
    if (ipMatches && ipMatches.length > 0) {
        const [_ip] = ipMatches
        ip = _ip
        // ip = '178.128.110.30'
        try {
            const res = await axios.get(`https://www.ip-adress.com/ip-address/ipv4/${ip}`)
            const html = res.data
            // 解析html
            const tblIn = html.split("<table class='vtable'>")[1].split("</table>")[0]
            const $ = cheerio.load(`<table class="vtable">${tblIn}</table>`)
            localLog("ip address info =>", $.text())
            const ths = $("th")
            const tds = $("td")

            for (let i = 0; i < ths.length; i += 1) {
                const thNode = $(ths[i])
                if (thNode.text() === "Country") {
                    region.country = $(tds[i]).text()
                }
                if (thNode.text() === "State") {
                    region.province = $(tds[i]).text()
                } else if (thNode.text() === "City") {
                    region.city = $(tds[i]).text()
                }
            }
        } catch (e) {
            console.log(e)
        }
    } else {
        ip = ipAddr
    }

    ctx.body = {
        Result: 1,
        success: true,
        code: 1,
        data: {
            ip,
            region,
        },
    } as CommonHttpResult<any>
}

/**
 * 接口: 代理获取图片
 * */
export const getImg = async (ctx: any) => {
    const { query } = ctx.request
    if (!query || !query.i) {
        ctx.send(200, {
            code: 0,
            message: "img must provide",
        })
        return
    }

    const imgUrl = decodeURIComponent(query.i)
    const options: HttpHeadOption = {
        path: imgUrl,
        headers: {},
        isSecure: imgUrl.indexOf("https") > -1,
    }

    // options.hostname = String(imgUrl).split('/').splice(0,3).join('/')
    const urlPcs = String(imgUrl).split("/")
    if (urlPcs.length > 2) {
        const [, , _hname] = urlPcs
        options.hostname = _hname
    }
    // options.headers.Host = options.hostname

    const resCont = await restUtil.get(options)

    // 3.response
    ctx.response.header["Access-Control-Allow-Origin"] = `http${options.isSecure ? "s" : ""}://${ctx.request.header.host
        }/`
    // Access-Control-Allow-Headers ,可根据浏览器的F12查看,把对应的粘贴在这里就行
    ctx.response.header["Access-Control-Allow-Methods"] = "*"

    const mimeType = mimeTransfer.getTypeByUrl(imgUrl)

    ctx.set("content-type", mimeType)
    ctx.set("Accept-Ranges", "bytes")
    ctx.set("Content-Length", Buffer.byteLength(resCont))
    ctx.set("Content-Disposition", `filename="${urlPcs[urlPcs.length - 1]}"`)
    ctx.response.status = 200
    ctx.response.body = resCont
}

/**
 * 接口: 第二种代理获取图片的方法
 * */
export const getImg2 = async (ctx: any) => {
    const { query, params } = ctx
    if ((query && query.i) || (params && params.imgUrl)) {
        console.log("no_params")
    } else {
        ctx.send(200, {
            code: 0,
            message: "img must provide",
        })
        return
    }

    let imgUrl = null
    if (query.i) {
        imgUrl = decodeURIComponent(query.i)
    } else {
        imgUrl = decodeURIComponent(params.imgUrl)
    }
    const options: HttpHeadOption = {
        path: imgUrl,
        headers: {},
        isSecure: imgUrl.indexOf("https") > -1,
    }

    // options.hostname = String(imgUrl).split('/').splice(0,3).join('/')
    const urlPcs = String(imgUrl).split("/")
    const [, , _hname] = urlPcs
    options.hostname = _hname
    // options.headers.Host = options.hostname

    const resCont = await restUtil.get(options)

    // 3.response
    const { host } = ctx.request.header
    const secureFlag = options.isSecure ? "s" : ""
    ctx.response.header["Access-Control-Allow-Origin"] = `http${secureFlag}://${host}/`
    // Access-Control-Allow-Headers ,可根据浏览器的F12查看,把对应的粘贴在这里就行
    ctx.response.header["Access-Control-Allow-Methods"] = "*"

    const mimeType = mimeTransfer.getTypeByUrl(imgUrl)

    ctx.set("content-type", mimeType)
    ctx.set("Accept-Ranges", "bytes")
    ctx.set("Content-Length", Buffer.byteLength(resCont))
    ctx.set("Content-Disposition", `filename="${urlPcs[urlPcs.length - 1]}"`)
    ctx.response.status = 200
    ctx.response.body = resCont
}

/**
 * 接口: 埋点提交
 * */
export const dotting = async (ctx: any) => {
    const { eventName, data, label, value } = ctx.request.body
    console.log("999999999999999: ", dataJSON.dottingUrl, ",", dataJSON.productInfo.dottingUrl)
    const res = await H5Tracer.invoke(
        eventName,
        {
            ...data,
        },
        {
            category: "VidNow_Category",
            appName: "VidNow",
            appVersion: "",
            packageName: "com.vd.vidnow",
            label,
            value,
        },
        dataJSON.dottingUrl
    )
    ctx.body = {
        Result: Number(res.data.Result),
        success: res.data.Result === 1,
        code: res.data.Result,
        data: res.data,
    } as CommonHttpResult<any>
}
type ShareBaseParam = {
    campaignPid: string
    shareNumber: string
}
/**
 * 旧有的[活动id+分享id]映射到新的[活动id+分享id]
 * */
function transferDnOld2NewCode(param: ShareBaseParam) {
    const { campaignPid, shareNumber } = param

    const rlt = {
        campaignPid,
        shareNumber,
    }
    const combine = `${campaignPid}_${shareNumber}`
    switch (combine) {
        // case "10098_1010":
        // case "10098_1014":
        // case "10098_1015":
        // case "10098_1021":
        //   //movie
        //  rlt.campaignPid = 10020
        //  rlt.shareNumber = 10114
        //   break;
        case "10099_1009":
            // downloadShare_earning_en
            rlt.campaignPid = "10020"
            rlt.shareNumber = "10105"
            break
        case "10099_1016":
            // downloadShare_earning_hi
            rlt.campaignPid = "10020"
            rlt.shareNumber = "10106"
            break
        case "10099_1017":
            // downloadShare_earning_id
            rlt.campaignPid = "10020"
            rlt.shareNumber = "10107"
            break
        case "10099_1018":
            // downloadShare_normal_en
            // downloadShare_earning_id
            rlt.campaignPid = "10020"
            rlt.shareNumber = "10108"
            break
        case "10098_1010":
            // videoShare_earning_en
            rlt.campaignPid = "10020"
            rlt.shareNumber = "10111"
            break
        case "10098_1014":
            // videoShare_earning_hi
            rlt.campaignPid = "10020"
            rlt.shareNumber = "10112"
            break
        case "10098_1015":
            // videoShare_earning_id
            rlt.campaignPid = "10020"
            rlt.shareNumber = "10113"
            break
        case "10098_1021":
            // videoShare_normal_en
            rlt.campaignPid = "10020"
            rlt.shareNumber = "10114"
            break
        default:
            // // downloadShare_earning_id
            // rlt.campaignPid = 10020
            // rlt.shareNumber = 10107
            break
    }
    return rlt
}

/**
 * 获取分享信息
 * */
function getShareInfo(
    campaignPid: string,
    shareNumber: string
): {
    campaignInfo: CampaignInfo | undefined
    shareInfo: ShareInfo | undefined
} {
    let campaignInfo: CampaignInfo | undefined
    let shareInfo: ShareInfo | undefined
    const { productInfo } = dataJSON
    Object.keys(productInfo.campaigns_Dic).forEach((cKey) => {
        const campaignItem = productInfo.campaigns_Dic[cKey]
        if (campaignItem.campaignPid === campaignPid) {
            campaignInfo = campaignItem as CampaignInfo
            Object.keys(campaignItem.campaign_shares_Dic).forEach((csKey) => {
                const shareItem = campaignItem.campaign_shares_Dic[csKey]
                if (shareItem.shareNumber === shareNumber) {
                    shareInfo = shareItem as ShareInfo
                }
            })
        }
    })

    return {
        campaignInfo,
        shareInfo,
    }
}
