import postUtil from "../../utils/post"
import localLog from "../../utils/logUtil"
import cryptoUtil from "../../utils/crypto"
import { CommonHttpResult } from "../../utils/http"

const jwt = require("jsonwebtoken")

const cfgFileName = process.env.NODE_ENV_CFG || "development"
// eslint-disable-next-line import/no-dynamic-require
const dataJSONFromCfg = require(`../../config/${cfgFileName}.json`)
localLog(
    "[server]-[config]-[db]-p2 env ",
    String(process.env.NODE_ENV),
    ", cfgFileName ",
    cfgFileName
)
const fs = require("fs")
const path = require("path")

/** 生成token */
function generateToken(data: any) {
    const created = Math.floor(Date.now() / 1000)
    const cert = fs.readFileSync(path.join(__dirname, "../../config/user.pri.pem"))
    // console.log('1222222222222222', cert)
    const token = jwt.sign(
        {
            data, // 自定义字段
            exp: created + 60 * 30, // 过期时间 30 分钟
            iat: created, // 创建时间
        },
        cert,
        { algorithm: "RS256" }
    )
    return token
}

/** 校验token */
function verifyToken(token: string) {
    const cert = fs.readFileSync(path.join(__dirname, "../../config/user.pub.pem")) // 公钥 可以自己生成
    let res
    try {
        const result = jwt.verify(token, cert, { algorithms: ["RS256"] }) || {}
        // let _id = result.data
        // let _date = result.exp
        // let _creatDate = result.iat
        const { exp = 0 } = result
        const current = Math.floor(Date.now() / 1000)
        if (current <= exp) {
            res = true
        } else {
            res = false
        }
    } catch (e) {
        res = false
        // console.log('222222222222222222 verifyToken not ok.', res)
    }
    return res
}

class UserController {
    static async login(ctx: any) {
        const dataStr = await postUtil.getPostData(ctx)
        const data = JSON.parse(dataStr)

        // 从配置文件中读取uname和pwd
        const sucUname = dataJSONFromCfg.username
        const sucPassword = dataJSONFromCfg.password
        const { md5random } = dataJSONFromCfg

        const md5SucPassword = cryptoUtil.Encrypt(Buffer.from(sucPassword), md5random) // md5解密pwd
        localLog("password", data.pwd, ",", md5SucPassword)

        if (sucUname === data.uname && md5SucPassword === data.pwd) {
            localLog(String(sucUname === data.uname), String(md5SucPassword === data.pwd))

            ctx.body = {
                Result: 1,
                success: true,
                code: 1,
                data: {
                    token: generateToken(sucUname),
                },
            } as CommonHttpResult<any>
        } else {
            ctx.body = {
                Result: 0,
                success: false,
                code: 0,
                message: "Access denied.",
            } as CommonHttpResult<any>
        }
    }

    /** 仅用于内部校验 */
    static async verify(ctx: any) {
        const { token } = ctx.req.headers
        const data = verifyToken(token)
        if (data) {
            return {
                Result: 1,
                success: true,
                code: 1,
                data: true,
            } as CommonHttpResult<boolean>
        }

        return {
            Result: 0,
            success: false,
            code: 0,
            message: "token is exceeded",
        } as CommonHttpResult<any>
    }
}

export default UserController
