import { sha256 } from "js-sha256"
import axios from "axios"

type DotingBaseOption = {
    category: string
    appName: string
    appVersion: string
    packageName: string
    label: string
    value: string
}
// const supportSendBeacon = typeof navigator.sendBeacon === 'function';
const H5Tracer = {
    // supportSendBeacon,
    uaObj: {
        appName: "",
        osType: "",
        osVersion: "",
        appVersion: "",
    },
    staticData: {
        country: "",
        language: "",
    },
    /**
     * 马上执行埋点数据提交
     * @param { 事件名 } eventName
     * @param { 事件数据(userId/deviceId可以不用传) } data
     * @param { 事件类型 } category
     * @param { 使用所属应用 } appName
     * @param { 使用所属应用的版本 } appVersion
     * @param { 使用所属应用对应包名 } packageName
     */
    invoke(eventName: string, data: any, options: DotingBaseOption, actionUrlPrefix: string) {
        const { category, appName, appVersion, packageName, label, value } = options || {}
        const dtNow = Date.now()
        // 1. 构造对象
        const newData = {
            ...data,
        }
        newData.userId = data.dot_userId
        newData.deviceId = data.dot_deviceId
        newData.ua = data.userAgent
        newData.url = data.href

        const msg = {
            events: [
                {
                    sessionid: `sid.${String(dtNow)}${Math.floor(Math.random() * 1000)}`,
                    userid: newData.userId,

                    deviceid: newData.deviceId,

                    country: newData.country,
                    language: newData.language,

                    isp: "",
                    idfa: `idfa.${newData.userId}`,
                    // bId: packageName,
                    bid: packageName,

                    appName,
                    appVersion,
                    osType: newData.osType,
                    osVersion: newData.osVersion,

                    timestamp: dtNow,
                    data: {
                        category,
                        event: eventName,
                        time: dtNow,
                        label: label === undefined ? "" : label,
                        value: value === undefined ? "" : value,
                        index: 1,
                        properties: newData,
                    },
                },
            ],
            version: 1,
        }

        // 2. 计算sign
        let sign = ""
        const msgStr = JSON.stringify(msg)

        const myBuff = Buffer.from(msgStr, "utf-8")
        if (myBuff.length > 128) {
            const newArr = []
            const buf1 = myBuff.slice(0, 64)
            for (let i = 0; i < buf1.length; i += 1) {
                newArr.push(buf1[i])
            }
            const buf2 = myBuff.slice(myBuff.length - 64)
            for (let i = 0; i < buf2.length; i += 1) {
                newArr.push(buf2[i])
            }
            sign = sha256.update(newArr).hex()
        } else {
            sign = sha256.update(myBuff).hex()
        }

        // 3. 执行ajax
        const ajaxUrl = `${actionUrlPrefix}/report/log/async`
        const ajaxData = {
            topic: "ANALYSE",
            msg: msgStr,
            sign,
        }
        // 3. 执行ajax
        // if (preferBeacon && supportSendBeacon) {
        //   navigator.sendBeacon(ajaxUrl, JSON.stringify(ajaxData));
        // } else {
        return axios.post(ajaxUrl, ajaxData, {
            headers: {
                mimeType: "application/json",
            },
        })

        // ajaxTool.ajax({
        //   method: 'post',
        //   url: ajaxUrl,
        //   data: ajaxData,
        //   mimeType: 'application/json'
        // });
        // }
    },
}

export default H5Tracer
