import http from "http"
import https from "https"

const Q = require("q")

// import { localLog } from "./logUtil";
export type HttpHeadOption = {
    method?: string
    port?: number
    hostname?: string
    path: string
    headers: any
    isSecure: boolean
}
/**
 * 额外的https/http请求
 */
export default class restUtil {
    /**
     * HOW TO Make an HTTP Call - GET
     *
     *  options for GET
     *  var optionsget = {
     *      hostname : 'api.t.sina.com.cn',
     *      path : '//provinces.json' // the rest of the url with parameters if needed
     *  };
     */
    static get(optionsget: HttpHeadOption) {
        return restUtil.request(optionsget, "GET", null, !!optionsget.isSecure)
    }

    static post(optionspost: HttpHeadOption, jsonObject: any) {
        if (optionspost.isSecure) {
            return restUtil.request(optionspost, "POST", jsonObject, true)
        }
        return restUtil.request(optionspost, "POST", jsonObject, false)
    }

    /**
     * HOW TO Make an HTTPs Call - GET
     *
     *  options for GET
     *  var optionsget = {
     *      hostname : 'graph.facebook.com',
     *      path : '/youscada' // the rest of the url with parameters if needed
     *  };
     */
    static secureGet(optionsget: HttpHeadOption) {
        return restUtil.request(optionsget, "GET", null, true)
    }

    /**
     * HOW TO Make an HTTPs Call - POST
     *
     *  options for POST
     *  var optionspost = {
     *      hostname : 'graph.facebook.com',
     *      path : '/youscada/feed?access_token=your_api_key'
     *  };
     */
    static securePost(optionspost: HttpHeadOption, jsonObject: any) {
        return restUtil.request(optionspost, "POST", jsonObject, true)
    }

    /**
     *  nodejs请求图片等大文件
     */
    static request = (
        options: HttpHeadOption,
        method: string,
        jsonObject: any,
        isSecure: boolean
    ) => {
        if (!(options && options.hostname && options.path)) {
            throw new Error("Omit property : 'hostname' or 'path'.")
        }
        const newOptions: HttpHeadOption = {
            path: "",
            headers: {},
            isSecure: true,
        }
        Object.assign(newOptions, options)
        newOptions.method = method || "GET"
        newOptions.port = options.port || 80
        if (Object.prototype.toString.call(jsonObject) === "[object Object]") {
            const jsonStr = JSON.stringify(jsonObject)
            const postheaders = {
                "Content-Type": "application/json",
                "Content-Length": Buffer.byteLength(jsonStr, "utf8"), // , 'utf8'
            }
            newOptions.headers = postheaders
        }

        let protocol: any = http

        if (isSecure) {
            newOptions.port = 443
            protocol = https
        }

        const deferred = Q.defer()
        let body: Buffer | undefined
        // do the POST call
        // localLog('kkkkkkkkkkkkkkk', JSON.stringify(options), String(isSecure))
        const request = protocol.request(options, (res: http.IncomingMessage) => {
            res.on("data", (trunk: Buffer) => {
                if (!body) {
                    body = trunk
                } else {
                    body = Buffer.concat([body, trunk], body.length + trunk.length)
                }
            })
                .on("end", () => {
                    deferred.resolve(body)
                })
                .on("errer", (err: any) => {
                    deferred.reject(err)
                })
        })

        // write the json data
        if (options.method === "POST") {
            request.write(jsonObject)
        }

        request.end()

        request.on("error", (e: any) => {
            deferred.reject(e)
        })

        return deferred.promise
    }
}
